/*
 * Copyright (c) 2016, WSO2 Inc. (http://wso2.com) All Rights Reserved.
 * <p>
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.ballerinalang.model.types;

import org.ballerinalang.model.values.BMap;
import org.ballerinalang.model.values.BString;
import org.ballerinalang.model.values.BValue;

/**
 * {@code BMapType} represents a type of a map in Ballerina.
 * <p>
 * Maps are defined using the map keyword as follows:
 * map mapName
 * <p>
 * All maps are unbounded in length and support key based indexing.
 *
 * @since 0.8.0
 */
public class BMapType extends BType implements BIndexedType {

    private BType elementType;

    /**
     * Create a type from the given name.
     *
     * @param typeName string name of the type
     */
    BMapType(String typeName, BType elementType, String pkgPath) {
        super(typeName, pkgPath, BMap.class);
        this.elementType = elementType;
    }

    public BType getElementType() {
        return elementType;
    }

    @Override
    public <V extends BValue> V getZeroValue() {
        return null;
    }

    @Override
    public <V extends BValue> V getEmptyValue() {
        return (V) new BMap<BString, V>();
    }

    @Override
    public TypeSignature getSig() {
        return new TypeSignature(TypeSignature.SIG_REFTYPE, TypeEnum.MAP.getName());
    }

    @Override
    public int getTag() {
        return TypeTags.MAP_TAG;
    }
}
